@@ -26,11 +26,13 @@
 /* for audio */
 #define AUDIO FALSE
 
-/* syntax flags to be used for REGEX matching */
-#define XLM_REGEX_SYNTAX RE_SYNTAX_EMACS
-
-
-
 /* do we need the LIBIMPORT ? This toggles whether the local or
    system wide include files are being used. */
 #undef NEED_IMPORT 
+
+/* 
+   These are the variables to check which files to include at
+   compiletime for REGEX functionality -- we need all these to
+   catch all possible setups...
+*/
+#undef NEED_OWN_REGEX_H
