@@ -15,8 +15,7 @@
 
   - getopt_long_only()
 
-  - all REGEXP functions from rx.h - rx.c is only built into libimport.a if it couldn't
-  be found on the system to keep the Xlogmaster as small as possible
+  - all REGEXP functions from the GNU rx-1.9a
 
 
 */
@@ -35,26 +34,62 @@
 */
 #include "getopt.h"
 
-/* 
-   include file for the REGEXP functions from the local system if possible,
-   otherwise use our local version of it.
-*/
-#include "rx.h"
-
 #else
 /* 
    include file for proper GNUish commandline parsing 
 */
 #include <getopt.h>
 
-/* 
-   include file for the REGEXP functions from the local system if possible,
-   otherwise use our local version of it.
+#endif /* NEED_IMPORT */
+
+/*
+  Here we include the REGEX things... due to lots of different
+  implementations for these functions we need to have a certain
+  complexity...
+*/
+
+#if NEED_OWN_REGEX_H
+
+/*
+  Take our own include file...
+*/
+#include "rx/regex.h"         
+
+#else
+
+#if HAVE_RXPOSIX_H
+/*
+  Do we have a system wide "rxposix.h" file ?
+*/
+#include <rxposix.h>
+
+#else 
+#if HAVE_RX_RX_H
+/*
+  ...otherwise: do we have a system wide "rx/rx.h" ?
+*/
+#include <rx/rx.h>
+#else 
+#if HAVE_RX_H
+/*
+  ...otherwise: do we have a system wide "rx.h" ?
 */
 #include <rx.h>
+#else
+#if HAVE_REGEX_H
+/*
+  ..otherwise: do we have a system wide "regex.h" file ?
+*/
+#include <regex.h>
 
-#endif
+#endif /* HAVE_REGEX_H */
+#endif /* HAVE_RX_H */
+#endif /* HAVE_RX_RX_H */
+#endif /* HAVE_RXPOSIX_H */
 
 
+/*     +++---+++---+++    */
+#endif /* NEED_OWN_REGEX_H */
+
 
 #endif /* IMPORT_H */
