@@ -3,7 +3,7 @@
   alert.cc
 
   notifying subroutines for xlogmaster.cc
-  Copyright (C) 1998 Georg C. F. Greve
+  Copyright (C) 1998,1999 Georg C. F. Greve
   This is a GNU program
   
   This program is free software; you can redistribute it and/or modify
@@ -217,7 +217,8 @@ create_notice_window()
   gtk_entry_set_text(GTK_ENTRY(text_field), " ");
   gtk_entry_set_editable (GTK_ENTRY(text_field),FALSE);
   gtk_tooltips_set_tip(tooltips, text_field, "... the line that triggered the alarm ...", "... the line that triggered the alarm ...");
-  gtk_container_add(GTK_CONTAINER(scrolled_window), text_field);
+  gtk_scrolled_window_add_with_viewport 
+    (GTK_SCROLLED_WINDOW(scrolled_window), text_field);
   gtk_widget_set_usize(text_field, 1200, -1);
   gtk_widget_show(text_field);
   gtk_widget_show(scrolled_window);
@@ -231,7 +232,8 @@ create_notice_window()
   notice_list =  gtk_list_new();
   gtk_widget_show(notice_list);
   gtk_list_set_selection_mode(GTK_LIST(notice_list), GTK_SELECTION_BROWSE);
-  gtk_container_add(GTK_CONTAINER(notice_scrolled_window), notice_list);
+  gtk_scrolled_window_add_with_viewport 
+    (GTK_SCROLLED_WINDOW(notice_scrolled_window), notice_list);
   gtk_signal_connect (GTK_OBJECT (notice_list), "select_child",
 		      GTK_SIGNAL_FUNC (notice_item_select), (gpointer) notice_list );
 
@@ -248,9 +250,10 @@ create_notice_window()
 /*}}}*/
 
 /*{{{  notice window subroutines  */
-void notice_taken(GtkWidget *widget, gpointer *pointer){
+void 
+notice_taken(GtkWidget *, gpointer *)
+{
   gtk_widget_hide(notice_dialog);
-  GtkWidget* active_item;
   
   gint children = g_list_length( (GList*) GTK_LIST(notice_list)->children );
 
@@ -261,18 +264,26 @@ void notice_taken(GtkWidget *widget, gpointer *pointer){
   }
 
 };
-void notice_item_select(GtkList* list, GtkWidget* child){
+void 
+notice_item_select(GtkList*, GtkWidget*)
+{
   GtkWidget* active_item = (GtkWidget*) GTK_LIST(notice_list)->selection->data;
   gchar* string = (gchar*) gtk_object_get_data( GTK_OBJECT( active_item ), LINE_TAG );
   gtk_entry_set_text(GTK_ENTRY(text_field), string);
+  gtk_entry_set_max_length(GTK_ENTRY(text_field), strlen(string));
+  gtk_widget_show(text_field);
   gtk_entry_set_position(GTK_ENTRY(text_field), 0);
 }
-void destroy_notice_entry(GtkWidget *widget, gpointer *data){
+void 
+destroy_notice_entry(GtkWidget *widget, gpointer *)
+{
   gchar* string = (gchar*) 
     gtk_object_get_data( GTK_OBJECT( widget ), LINE_TAG );
   delete string;
 }
-void notice_alert(gchar* name, gchar* line){
+void 
+notice_alert(gchar* name, gchar* line)
+{
   int x = 0;
   int a = 0;
   gchar* string = new gchar[strlen(line)+1];
