@@ -3,7 +3,7 @@
   file.cc
 
   file access subroutines for xlogmaster.cc
-  Copyright (C) 1998 Georg C. F. Greve
+  Copyright (C) 1998,1999 Georg C. F. Greve
   This is a GNU program
   
   This program is free software; you can redistribute it and/or modify
@@ -41,31 +41,37 @@
 
 /*}}}*/
 
-/*{{{  Read Configuration  */
+/*{{{  Read Configuration file  */
 /*
-  This function reads a Xlogmaster-Style configuration file
+  This routine reads a configuration file & returns
+  a fully qualified, completely initialized NULL
+  terminated array of Log class objects with filters
+  and everything else.
   
-  The parameter is the filename (absolute !!) and the results are
-  given in the "fillup-structure"
-
+  The parameter is the absolute filename of the configuration
+  file to be read - the return value is the pointer to
+  the structure that has been filled up.
 */
-int 
-read_configuration(char* fname)
+Log**
+read_configuration_file(gchar* filename)
 {
-  int fd = open(fname, O_RDONLY);
-  if ( fd == -1 ) return FALSE;
+  Log** array = NULL;
+
+  gint fd = open(filename, O_RDONLY);
+  if ( fd == -1 ) return array;
   fstat( fd,  &status );
   int length = (int) status.st_size;
   char* buffer = new char[length+2];
 
-  int got = (int) read(fd, buffer, length);
+  gint got = (gint) read(fd, buffer, length);
   close(fd);
-  if ( got != length ){
-    delete buffer;
-    return FALSE;
-  }
+  if ( got != length )
+    {
+      delete buffer;
+      return array;
+    }
   buffer[length] = 0; // Make it zero terminated
-
+  
   /*
     And now start browsing the file
   */
@@ -85,23 +91,22 @@ read_configuration(char* fname)
       
       buffer[x] = 0;
       a = 0;
-      gint result = -1;
+      gint result = REG_NOMATCH;
 
       while ( mode_regex[a] != NULL &&
-	      result < 0 )
+	      result == REG_NOMATCH )
 	{
 	  
-	  result = re_search( *(mode_regex[a]),
-			      line,
-			      strlen(line),
-			      0,
-			      strlen(line),
-			      NULL);
+	  result = regexec( *(mode_regex[a]),
+			    line,
+			    0,
+			    NULL,
+			    0);
 	  
-	  if ( result < 0 ) a++;
+	  if ( result == REG_NOMATCH ) a++;
 	}
       
-      if ( result >= 0 )
+      if ( result == 0 )
 	{
 	  /*
 	    Line is a mode line
@@ -110,49 +115,56 @@ read_configuration(char* fname)
 	  gint new_mode = mode_flags[a];
 	  
 	  guint index = searchfor('{', line, 0);
-	  if ( index >= strlen(line) ){
-	    free_regex_patterns();	    
-	    delete buffer;
-	    return TRUE;
-	  }
+	  if ( index >= strlen(line) )
+	    {
+	      free_regex_patterns();	    
+	      delete buffer;
+	      return array;
+	    }
 	  index++;
 	
 	  /*
 	    retrieve filename
 	  */
 	  gchar* new_filename = getstring(',', line, index);
-	  if ( new_filename == NULL ){
-	    free_regex_patterns();
-	    delete buffer;
-	    return TRUE;
-	  }
+	  if ( new_filename == NULL )
+	    {
+	      free_regex_patterns();
+	      delete buffer;
+	      return array;
+	    }
 	  index += strlen(new_filename)+1;
 	  new_filename = clean_string(new_filename);
 
 	  /*
-	    retrieve time interval
+	    retrieve time interval & delay information of value
+	    interval[/delay]
 	  */
 	  gint new_interval = 0;
+	  gint new_delay = 6;
 	  gchar* new_int_string = getstring(',', line, index);
-	  if ( new_int_string == NULL ){
-	    delete new_filename;
-	    free_regex_patterns();
-	    delete buffer;
-	    return TRUE;
-	  }
+	  if ( new_int_string == NULL )
+	    {
+	      delete new_filename;
+	      free_regex_patterns();
+	      delete buffer;
+	      return array;
+	    }
 	  index += strlen(new_int_string)+1;
 	  new_int_string = clean_string(new_int_string);
-	  gint y = 0;
-	  while ( new_int_string[y] != 0 )
+	  guint time_index = searchfor('/', new_int_string, 0);
+	  if ( time_index < strlen(new_int_string) )
 	    {
-	      gint c = new_int_string[y] - '0';
-	      if ( c >= 0 && c <= 9 )
-		{
-		  new_interval *= 10;
-		  new_interval += c;
-		}
-	      y++;
+	      /* separate values */
+	      new_int_string[time_index] = 0;
+	      time_index++;
+	      
+	      /* read delay value */
+	      gchar* delay_string = (gchar*) (&(new_int_string[time_index]));
+	      istrstream(delay_string, strlen(delay_string)) >> new_delay;
 	    }
+	  /* now read interval value */
+	  istrstream(new_int_string, strlen(new_int_string)) >> new_interval;
 	  if ( new_interval == 0 ||
 	       strlen(new_int_string) == 0 ) new_interval = INTERVAL;
 	  delete new_int_string;
@@ -161,12 +173,13 @@ read_configuration(char* fname)
 	    retrieve buttontext
 	  */
 	  gchar* new_buttontext = getstring(',', line, index);
-	  if ( new_buttontext == NULL ){
-	    delete new_filename;
-	    free_regex_patterns();
-	    delete buffer;
-	    return TRUE;
-	  }
+	  if ( new_buttontext == NULL )
+	    {
+	      delete new_filename;
+	      free_regex_patterns();
+	      delete buffer;
+	      return array;
+	    }
 	  index += strlen(new_buttontext)+1;
 	  new_buttontext = clean_string(new_buttontext);
 	  
@@ -174,31 +187,97 @@ read_configuration(char* fname)
 	    retrieve helptext
 	  */
 	  gchar* new_help = getstring('}', line, index);
-	  if ( new_help == NULL ){
-	    delete new_buttontext;
-	    delete new_filename;
-	    free_regex_patterns();
-	    delete buffer;
-	    return TRUE;
-	  }
+	  if ( new_help == NULL )
+	    {
+	      delete new_buttontext;
+	      delete new_filename;
+	      free_regex_patterns();
+	      delete buffer;
+	      return array;
+	    }
 	  index += strlen(new_help)+1;
 	  new_help = clean_string(new_help);
-	  if ( strlen(new_help) == 0 ){
-	    delete new_help;
-	    new_help = new char[strlen(new_filename)+1];
-	    strcpy(new_help, new_filename);
-	  }
+	  if ( strlen(new_help) == 0 )
+	    {
+	      delete new_help;
+	      new_help = new char[strlen(new_filename)+1];
+	      strcpy(new_help, new_filename);
+	    }
 
 	  /*
 	    and now add the entry...
 	  */
-	  add_mode_entry(new_mode, new_filename, new_help, new_buttontext, new_interval);
+	  array = add_log_entry( array,
+				 new_mode,
+				 new_filename,
+				 new_help,
+				 new_buttontext,
+				 NULL,
+				 new_interval,
+				 new_delay );
 
+	  /*
+	    and now kill the temporaries again
+	  */
+	  delete new_filename;
+	  delete new_help;
+	  delete new_buttontext;
+
+	}
+      else if ( regexec( plugin_regex,
+			 line,
+			 0,
+			 NULL,
+			 0 ) == 0 )
+	{
+
+	  /*
+	    Line is a PLUGIN line
+	  */
+	  
+	  gchar* new_filter_line = new gchar[strlen(line)];
+
+	  gint x = 0;
+	  gint start = (gint) searchfor('{', line, 0) + 1;
+	  gint stop = strlen(line);
+	  while ( stop >= 0 &&
+		  line[stop] != '}' ) stop--;
+	  
+	  while ( start < stop )
+	    {
+	      new_filter_line[x] = line[start];
+	      x++;
+	      start++;
+	    }
+	  new_filter_line[x] = 0;
+
+	  Log* element = NULL;
+	  x = 0;
+	  
+	  while ( array != NULL && array[x]!= NULL )
+	    {
+
+	      element = array[x];
+	      x++;
+
+	    }
+	  
+	  if ( element != NULL )
+	    {
+	      
+	      if ( element->plugin != NULL )
+		delete element->plugin;
+	      
+	      element->plugin = new_filter_line;
+	      
+	    }
+	  
 	}
       else
 	{
+	  
 	  /*
-	    Line is NOT a mode line, so we need to check whether
+	    Line is no MODE or PLUGIN line, so we need to check whether
 	    it is a filter line...
 	  */
 
@@ -208,19 +287,18 @@ read_configuration(char* fname)
 	  while ( filter_regex[a] != NULL )
 	    {
 	      
-	      result = re_search( *(filter_regex[a]),
-				  line,
-				  strlen(line),
-				  0,
-				  strlen(line),
-				  NULL);
+	      result = regexec( *(filter_regex[a]),
+				line,
+				0,
+				NULL,
+				0 );
 
 	      /*
 		if we have a match add the according flag to
 		the filter flags
 	      */
 	      
-	      if ( result >= 0 )
+	      if ( result == 0 )
 		{
 		  new_filter_mode |= filter_flags[a];
 		}
@@ -310,7 +388,16 @@ read_configuration(char* fname)
 		And now add the filter entry to the current
 		log entry
 	      */
-	      add_filter_entry(new_filter_string, new_filter_mode, new_filter_execline);
+	      add_filter_to_last( array,
+				  new_filter_string,
+				  new_filter_mode,
+				  new_filter_execline );
+	      
+	      delete new_filter_string;
+	      if ( new_filter_execline != NULL )
+		{
+		  delete new_filter_execline;
+		}
 	      
 	    } 
 	}
@@ -329,179 +416,40 @@ read_configuration(char* fname)
     }
   
   free_regex_patterns();
-  delete buffer;
-  return TRUE;
+  delete buffer;  
+
+  return array;
 }
-/*
-  This routine adds a new entry to the array that
-  is used to set up the real Log Class entries
-
-  The parameters are the values of the entries that are to be added to the
-  array.
-
-*/
-void
-add_mode_entry(gint new_mode, gchar* new_filename, gchar* new_help, gchar* new_buttontext, gint new_interval)
-{
-  if ( fillups == 0 )
-    {
-      fillups = 1;
-      mode = new int[fillups];
-      filename = new char* [fillups];
-      help = new char* [fillups];
-      buttontext = new char* [fillups];
-      interval = new int[fillups];
-      commandline = new char**[fillups];
-      filter = new char**[fillups];
-      filter_exec = new char**[fillups];
-      filter_mode = new int*[fillups];
-      mode[fillups-1] = new_mode;
-      filename[fillups-1] = new_filename;
-      help[fillups-1] = new_help;
-      buttontext[fillups-1] = new_buttontext;
-      interval[fillups-1] = new_interval;
-      commandline[fillups-1] = NULL;
-      filter[fillups-1] = NULL;
-      filter_exec[fillups-1] = NULL;
-      filter_mode[fillups-1] = 0;
-    }
-  else
-    {
-      int *tmp_mode= mode;                 
-      char **tmp_filename = filename;
-      char **tmp_help = help;  
-      char **tmp_buttontext = buttontext;  
-      int *tmp_interval = interval;
-      char ***tmp_commandline = commandline;
-      char ***tmp_filter = filter;
-      char ***tmp_filter_exec = filter_exec;
-      int **tmp_filter_mode = filter_mode;
-      fillups++;
-      mode = new int[fillups];
-      filename = new char* [fillups];
-      help = new char* [fillups];
-      buttontext = new char* [fillups];
-      interval = new int[fillups];
-      commandline = new char**[fillups];
-      filter = new char**[fillups];
-      filter_exec = new char**[fillups];
-      filter_mode = new int*[fillups];
-      for ( int i = 0 ; i < fillups-1 ; i++ )
-	{
-	  mode[i]= tmp_mode[i];                 
-	  filename[i] = tmp_filename[i];
-	  help[i] = tmp_help[i];  
-	  buttontext[i] = tmp_buttontext[i];  
-	  interval[i] = tmp_interval[i];
-	  commandline[i] = tmp_commandline[i];
-	  filter[i] = tmp_filter[i];
-	  filter_exec[i] = tmp_filter_exec[i];
-	  filter_mode[i] = tmp_filter_mode[i];
-	}
-      mode[fillups-1] = new_mode;
-      filename[fillups-1] = new_filename;
-      help[fillups-1] = new_help;
-      buttontext[fillups-1] = new_buttontext;
-      interval[fillups-1] = new_interval;
-      commandline[fillups-1] = NULL;
-      filter[fillups-1] = NULL;
-      filter_exec[fillups-1] = NULL;
-      filter_mode[fillups-1] = 0;
-      delete tmp_mode;
-      delete tmp_filename;
-      delete tmp_help;
-      delete tmp_buttontext;
-      delete tmp_interval;
-      delete tmp_commandline;
-      delete tmp_filter;
-      delete tmp_filter_exec;
-      delete tmp_filter_mode;
-    }
-}
-/*
-  Adds another filter entry to entry that is currently being
-  read
-*/
-void
-add_filter_entry(gchar* _string, gint _mode, gchar* _execline)
-{
-  if ( fillups < 1 ) return;
-
-  if ( filter[fillups-1] == NULL )
-    {
-      filter[fillups-1] = new gchar*[2];
-      filter_exec[fillups-1] = new gchar*[2];
-      filter_mode[fillups-1] = new gint[2];
-
-      (filter[fillups-1])[0] = _string;
-      (filter_exec[fillups-1])[0] = _execline;
-      (filter_mode[fillups-1])[0] = _mode;
-
-      (filter[fillups-1])[1] = NULL;
-      (filter_exec[fillups-1])[1] = NULL;
-      (filter_mode[fillups-1])[1] = 0;
-    }
-  else 
-    {
-      gint x = 0;
-      while ( (filter[fillups-1])[x] != NULL ) x++;
-
-      char** tmp_new_filter = new char*[x+2];
-      char** tmp_new_filter_exec = new char*[x+2];
-      int* tmp_new_filter_mode = new int[x+2];
-
-      x = 0;
-      while ( (filter[fillups-1])[x] != NULL )
-	{
-	  tmp_new_filter[x] = (filter[fillups-1])[x];
-	  tmp_new_filter_exec[x] = (filter_exec[fillups-1])[x];
-	  tmp_new_filter_mode[x] = (filter_mode[fillups-1])[x];
-	  x++;
-	}
-
-      delete (filter[fillups-1]);
-      delete (filter_exec[fillups-1]);
-      delete (filter_mode[fillups-1]);
-
-      (filter[fillups-1]) = tmp_new_filter;
-      (filter_exec[fillups-1]) = tmp_new_filter_exec;
-      (filter_mode[fillups-1]) =  tmp_new_filter_mode;
-
-      (filter[fillups-1])[x] = _string;
-      (filter_exec[fillups-1])[x] = _execline;
-      (filter_mode[fillups-1])[x] = _mode;
-
-      (filter[fillups-1])[x+1] = NULL;
-      (filter_exec[fillups-1])[x+1] = NULL;
-      (filter_mode[fillups-1])[x+1] = 0;
-
-    }
-}
-
 /*}}}*/
 
 /*{{{  Matching subroutines  */
 
 // legalize filter mode, only one of the CLASS0 filters is allowed
-gint legalize(gint fmode, gint def){
+gint 
+legalize(gint fmode, gint def)
+{
   gint activated = 0;
   gint x = 0;
-  while ( class0_filters[x] != -1 ){
-    if ( fmode & class0_filters[x] ) activated++;
-    x++;
-  }
+  while ( class0_filters[x] != -1 )
+    {
+      if ( fmode & class0_filters[x] ) activated++;
+      x++;
+    }
 
-  if ( activated > 1 ){
-    /* leave class1 and special mode things untouched 
-       and delete all class0 */
-    fmode &= ( CLASS1_MASK |  SPECIAL_MODE_MASK ); 
-    fmode |= def; // set default
-  }
- 
+  if ( activated > 1 )
+    {
+      /* leave class1 and special mode things untouched 
+	 and delete all class0 */
+      fmode &= ( CLASS1_MASK |  SPECIAL_MODE_MASK ); 
+      fmode |= def; // set default
+    }
+  
   return fmode;
 }
 // tries to find "string" in "buffer" at position "x"
-int match(char* string, char* buffer, int x){
+int 
+match(char* string, char* buffer, int x)
+{
   int i = 0;
   while( string[i] != 0 )
     if ( string[i++] != buffer[x++] )
@@ -510,13 +458,17 @@ int match(char* string, char* buffer, int x){
   return TRUE;
 }
 // searches for char 'c' in buffer starting at x and returning position
-int searchfor(char c, char* buffer, int x){
+int 
+searchfor(char c, char* buffer, int x)
+{
   while( buffer[x] != c && buffer[x] != 0 ) x++;
   return x;
 }
 // tries to create string from x to position of 'delim'
 // returns NULL if out of bounds...
-char* getstring(char delim, char* buffer, int x){
+char* 
+getstring(char delim, char* buffer, int x)
+{
   int y = searchfor( delim, buffer, x);
   if ( buffer[y] == 0 ) return NULL;
   char* string = new char[y-x+1];
@@ -528,7 +480,9 @@ char* getstring(char delim, char* buffer, int x){
   string[y-x] = 0;
   return string;
 }
-char* clean_string(char* source){
+char* 
+clean_string(char* source)
+{
   if ( source == NULL ) return NULL;
   char* target = new char [strlen(source)+1];
   // eliminate leading and trailing spaces + all CR/LF's
@@ -563,22 +517,11 @@ create_regex_patterns()
   while ( all_string[x] != NULL )
     {
       
-      (*(all_regex[x])) = new re_pattern_buffer;
-      (*(all_regex[x]))->buffer = NULL;
-      (*(all_regex[x]))->allocated = 0;
-      (*(all_regex[x]))->translate = NULL;
-      (*(all_regex[x]))->re_nsub = 0;
-      (*(all_regex[x]))->se_params = NULL; 
-      (*(all_regex[x]))->syntax_parens = NULL;
-      (*(all_regex[x]))->start = NULL;
-      re_set_syntax( RE_SYNTAX_EMACS );
-      (*(all_regex[x]))->fastmap = NULL;
-      
-      re_compile_pattern(all_string[x], strlen(all_string[x]), (*(all_regex[x])));
-      
-      (*(all_regex[x]))->fastmap = new gchar[256];
-      re_compile_fastmap( (*(all_regex[x])) );
+      (*(all_regex[x])) = new regex_t;
       
+      regcomp( (*(all_regex[x])),
+	       all_string[x],
+	       REG_NOSUB | REG_NEWLINE );
       x++;
     }
   
@@ -594,7 +537,8 @@ free_regex_patterns()
   while ( all_string[x] != NULL )
     {
       
-      disassemble_regex(*(all_regex[x]));
+      regfree(*(all_regex[x]));
+      delete (*(all_regex[x]));
       (*(all_regex[x])) = NULL;
       
       x++;
@@ -619,12 +563,22 @@ write_configuration(char* fname)
   for ( int i = 0 ; i < syslogs ; i++ )
     {
       if ( entry[i]->mode == TAIL_FILE ) file << "TAIL{";
+      else if ( entry[i]->mode == RUN_FILE ) file << "RUN{";
       else if ( entry[i]->mode == CAT_FILE ) file << "CAT{";
       file << entry[i]->filename << ",";
-      file << entry[i]->interval << ",";
+      file << entry[i]->interval;
+      if ( entry[i]->mode == RUN_FILE )
+	file << "/" << entry[i]->delay;
+      file << ",";
       file << entry[i]->buttontext << ",";
       file << entry[i]->help << "}" << endl;
       
+      if ( entry[i]->plugin != NULL )
+	{
+	  file << "PLUGIN{";
+	  file << entry[i]->plugin << "}" << endl;
+	}
+      
       if ( entry[i]->filter != NULL )
 	{
 	  int a = 0;
@@ -711,3 +665,4 @@ write_configuration(char* fname)
   return TRUE;
 }
 /*}}}*/
+
