@@ -2,8 +2,8 @@
 
   The Xlogmaster
   
-  a GTK+ based program to keep track of logfiles & devices
-  Copyright (C) 1998 Georg C. F. Greve
+  a GTK+ based program to keep track of logfiles & system status
+  Copyright (C) 1998,1999 Georg C. F. Greve
   This is a GNU program
   
   This program is free software; you can redistribute it and/or modify
@@ -63,7 +63,9 @@ main (int argc, char *argv[])
   init( argc, argv );
 
   /* create main window */
+  accel_group = gtk_accel_group_new ();
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_accel_group_attach (accel_group, GTK_OBJECT (window));
   gtk_widget_set_usize( GTK_WIDGET (window), width, height);
   gtk_signal_connect (GTK_OBJECT (window), "destroy",
 		      GTK_SIGNAL_FUNC (quit), NULL);
@@ -91,10 +93,13 @@ main (int argc, char *argv[])
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 				  GTK_POLICY_AUTOMATIC, 
 				  GTK_POLICY_AUTOMATIC );
-  gtk_container_add (GTK_CONTAINER (scrolled_window), buttonbox);
-  gtk_object_set_data( GTK_OBJECT(window), BUTTONBOX, (GtkWidget*) scrolled_window );
-  
+
   /* put box for buttons into window */
+  gtk_scrolled_window_add_with_viewport 
+    ( GTK_SCROLLED_WINDOW (scrolled_window), 
+      buttonbox);
+  gtk_object_set_data( GTK_OBJECT(window), BUTTONBOX, 
+		       (GtkWidget*) scrolled_window );
   gtk_box_pack_start (GTK_BOX(box), scrolled_window, FALSE, FALSE, 0);
   gtk_widget_show(scrolled_window);
 
@@ -107,9 +112,8 @@ main (int argc, char *argv[])
   /* do color stunts */
   color_init();
   
-  /* Now setup the data structures and create the buttons */
-  setup_logs();
-  create_buttons(); // ... as said
+  /* Now create the buttons */
+  create_buttons(); 
 
   /* PopUp Window */
   gtk_widget_show(buttonbox);
@@ -138,7 +142,7 @@ main (int argc, char *argv[])
 
 /*{{{  GTK toolkit subroutines  */
 gint
-delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+delete_event(GtkWidget *widget, GdkEvent *, gpointer data)
 {
   // we will attach this to all windows so the window managers "delete" signal
   // won't kill our windows (that's what returning TRUE means)
@@ -151,33 +155,34 @@ delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
   return (TRUE);
 }
 void 
-quit (GtkWidget *widget, gpointer *data)
+quit (GtkWidget *, gpointer *)
 {
-  if ( terse == TRUE ){
-    really_quit_button_pressed();
-    return;
-  }
+  if ( terse == TRUE )
+    {
+      really_quit_button_pressed();
+      return;
+    }
 
   GList* buttons = g_list_alloc();
   GList* functions = g_list_alloc();
   
   buttons = g_list_append(buttons,
-			  (gpointer) "\t   YES   \t");
+			  (gpointer) "    YES    ");
   functions = g_list_append(functions,
 			    (gpointer) really_quit_button_pressed );
   buttons = g_list_append(buttons,
-			  (gpointer) "\t   NO   \t");
+			  (gpointer) "    NO    ");
   functions = g_list_append(functions, NULL );
 
   if ( configuration_changed == TRUE )
     popup_dialog
       ("Really Quit ?", 
-       "\n\t     There are unsaved changes.     \t\n-\n\t     Do you really want to quit ?     \t\n", 
+       "\n      There are unsaved changes.      \n-\n      Do you really want to quit ?      \n", 
        buttons, functions);
   else
     popup_dialog
       ("Really Quit ?", 
-       "\n\t     Do you really want to quit ?     \t\n", 
+       "\n      Do you really want to quit ?      \n", 
        buttons, functions);
 }
 void 
@@ -270,148 +275,368 @@ create_textwidget()
  
 }
 /*
-  This function creates the buttons - it is called at startup and during reinitialisation
+  This function creates the buttons - 
+  it is called at startup and during reinitialisation
   when the user changed the config or loaded a new one...
+  
+  This routine is also responsible for counting the amount
+  of objects and putting the number into "syslogs" for the
+  other routines. The reason is simple: before the buttons
+  are up there is no routine needing the "syslogs" variable.
+  
+  no parameters or return values...
 */
 void 
 create_buttons()
 {
-  for ( int i = 0 ; i < syslogs ; i++ ){
-    entry[i]->button = gtk_button_new_with_label (entry[i]->buttontext);
-    gtk_signal_connect (GTK_OBJECT (entry[i]->button), "clicked",
-			GTK_SIGNAL_FUNC (button_pressed), (gpointer) i );
-    gtk_box_pack_start(GTK_BOX(buttonbox), entry[i]->button, FALSE, FALSE, 5);
-    gtk_tooltips_set_tip(tooltips, entry[i]->button, entry[i]->help,entry[i]->help);
-    gtk_widget_show(entry[i]->button);
-    entry[i]->style = gtk_style_copy(stdstyle);
-    gtk_widget_set_style(entry[i]->button, entry[i]->style);
-    gtk_style_unref(entry[i]->style);
-  }
+  /* if there is no structure to display we have a problem... */
+  if ( entry == NULL )
+    {
+      cerr << PACKAGE << ": critical problem, no configuration could be" << endl;
+      cerr << "  found - this should never happen, please refer to manuals" << endl;
+      cerr << "  for help." << endl;
+      _exit(0);
+    }
+
+  /* count the amount of Log class objects */
+  syslogs = 0;
+  while ( entry[syslogs] != NULL ) syslogs++;
+
+  guint8 mods = 0;
+  guchar key = '1';
+  
+  for ( gint i = 0 ; i < syslogs ; i++ )
+    {
+
+      entry[i]->button = gtk_button_new_with_label (entry[i]->buttontext);
+
+      gtk_signal_connect (GTK_OBJECT (entry[i]->button), "clicked",
+			  GTK_SIGNAL_FUNC (button_pressed), (gpointer) i );
+
+      gtk_box_pack_start(GTK_BOX(buttonbox), 
+			 entry[i]->button, 
+			 FALSE, 
+			 FALSE, 
+			 5 );
+
+      gtk_tooltips_set_tip(tooltips, 
+			   entry[i]->button, 
+			   entry[i]->help,
+			   entry[i]->help );
+
+      gtk_widget_show(entry[i]->button);
+      
+      /* install the keyboard accelerators */
+      if ( mods < ( GDK_MOD1_MASK | 
+		    GDK_CONTROL_MASK ) )
+	{
+
+	  entry[i]->accel_key = key;
+	  entry[i]->accel_mods = mods;
+
+	  gtk_widget_add_accelerator ( entry[i]->button ,
+				       "clicked" ,
+				       accel_group ,
+				       entry[i]->accel_key ,
+				       entry[i]->accel_mods ,
+				       GTK_ACCEL_VISIBLE );
+	  
+	}
+
+      /* 
+	 determine the next keyboard accelerator 
+	 
+	 first numbers from '1'-'0', then
+	 CTRL+'1'-'0', then ALT+'1'-'0', then 
+	 ALT+CTRL+'1'-'0'
+      */
+      if ( key == '0' )
+	{
+	  
+	  if ( mods & GDK_CONTROL_MASK )
+	    {
+      	      if ( mods & GDK_MOD1_MASK )
+		{
+		  
+		  /* alright if this wasn't enough we stop
+		     assigning keyboard accelerators now */
+		  mods = ( GDK_MOD1_MASK | 
+			   GDK_CONTROL_MASK |
+			   GDK_SHIFT_MASK );  
+		}
+	      else
+		{
+		  mods |= GDK_MOD1_MASK;
+		}
+	      
+	      if ( ! mods & GDK_SHIFT_MASK )
+		{
+		  mods &= ~GDK_CONTROL_MASK;
+		}
+	    }
+	  else
+	    {
+	      mods |= GDK_CONTROL_MASK;
+	    }  
+	}
+      key++;
+      if ( key > '9' )
+	{
+	  key = '0';
+	}
+      
+      /* and now the final style things... */
+      entry[i]->style = gtk_style_copy(stdstyle);
+      gtk_widget_set_style(entry[i]->button, entry[i]->style);
+      gtk_style_unref(entry[i]->style);
+      
+    }
+  
   gtk_widget_grab_focus(entry[0]->button);
+  gtk_widget_realize(window);
+  
 }
-void button_pressed(GtkWidget *widget, gpointer *data){
+/*
+  Function to do a clean removal of the buttons...
+*/
+void
+destroy_buttons()
+{
+  gint i = 0;
+  
+  while ( entry != NULL && entry[i] != NULL )
+    {
+
+      /* clear alert for the button... */
+      remove_alert(i);
+
+      /* remove the button. */
+      if ( entry[i]->button != NULL )
+	{
+
+	  gtk_widget_remove_accelerator ( entry[i]->button ,
+					  accel_group ,
+					  entry[i]->accel_key ,
+					  entry[i]->accel_mods );
+
+	  gtk_widget_destroy( entry[i]->button );
+
+	}
+      entry[i]->button = NULL;
+      
+      i++;
+      
+    }
+  
+  gtk_widget_realize(window);
+}
+void 
+button_pressed(GtkWidget *, gpointer *data)
+{
   if ( display_logs == DISABLED ) return;
   int pressed = (int) data;       // Which button has been pressed ?
   deactivate();                        // deactivate old "Log" object
   active = pressed;                 // set new one
   activate();                           // and activate it
 }
-void request_about(){
+void 
+request_about()
+{
   gtk_window_position(GTK_WINDOW(about_dialog), GTK_WIN_POS_MOUSE);
   gtk_widget_show(about_dialog);
 }
-void about_ready(GtkWidget *widget, gpointer *pointer){
+void 
+about_ready(GtkWidget *, gpointer *)
+{
   gtk_widget_hide(about_dialog);
 };
 /*}}}*/
 
 /*{{{  Data management  */
-void 
-setup_logs()
+/*
+  This routine creates a 1:1 copy of a Log Class Data Array
+
+  The argument is the array to be copied, the return value
+  is the copy.
+*/
+Log**
+copy_data_array(Log** array)
 {
-  entry = new Log*[fillups];
 
-  for ( int i = 0 ; i < fillups ; i++ ){
-    entry[i] = new Log;
-    entry[i]->mode = mode[i];
-    entry[i]->filename = filename[i];
-    if ( help[i] != NULL ) entry[i]->help = help[i];
-    else{
-      entry[i]->help = new char[strlen(entry[i]->filename)];
-      strcpy(entry[i]->help, entry[i]->filename);
+  if ( array == NULL ) return NULL;
+  
+  gint i = 0;
+  Log** copy = NULL;
+  while ( array[i] != NULL )
+    {
+
+      copy = add_log_entry( copy,
+			    array[i]->mode,
+			    array[i]->filename,
+			    array[i]->help,
+			    array[i]->buttontext,
+			    array[i]->plugin,
+			    array[i]->interval,
+			    array[i]->delay);
+      
+      gint j = 0;
+      while ( array[i]->filter != NULL &&
+	      array[i]->filter[j] != NULL )
+	{
+	  
+	  add_filter_to_last( copy ,
+			      array[i]->filter[j]->string ,
+			      array[i]->filter[j]->mode ,
+			      array[i]->filter[j]->execline );
+
+	  j++;
+	}
+      
+      i++;
+      
     }
-    entry[i]->buttontext = buttontext[i];
-    entry[i]->interval = interval[i];
-    entry[i]->commandline = commandline[i];
-    entry[i]->tag = 0;
-    entry[i]->alert_tag = 0;
-    entry[i]->buffer = entry[i]->rbuffer = NULL;
+  
+  return copy;
+}
+/*
+  This function takes a data array of Log class objects and appends
+  one that gets initialized with the given parameters.
 
-    // chunk gives the blocksize of the read for the watchdog
-    // must be maxtext for anything else than TAIL, of course.
-    entry[i]->chunk = RDBLCK * entry[i]->interval;
-    if ( ( entry[i]->mode != TAIL_FILE ) ||
-	 ( entry[i]->chunk > maxtext) ) entry[i]->chunk = maxtext;
-
-    /*
-      Fillup the filter elements with some mind dizzying memory juggling
-    */
-
-    if ( filter[i] != NULL ){
-      char **f = filter[i];
-      char **fe = filter_exec[i];
-      int *m = filter_mode[i];
-
-      int x = 0;
-      while ( f[x] != NULL ) x++;
-      entry[i]->filter = new Filter*[x+1];
-      entry[i]->filterclass = NO_FILTER;
-
-      x = 0;
-      while  ( f[x] != NULL ) {
-	entry[i]->filter[x] = new Filter();
-	entry[i]->filter[x]->init( f[x], m[x] | COMPILE_REGEX );
-	delete f[x];
-	entry[i]->filter[x]->execline = fe[x];
-	gint fclass = determine_filter_class(entry[i]->filter[x]->mode);
-	entry[i]->filterclass |= fclass;
-	x++;
-      }
-      entry[i]->filter[x] = NULL; // NULL Termination is crucial...
-      sort_filters(entry[i]->filter);
-      delete f;
-      delete fe;
-      delete m;
+  The new array is then returned...
+*/
+Log**
+add_log_entry(Log** array, gint mode, const gchar* filename, const gchar* help, const gchar* buttontext, const gchar* plugin, gint interval, gint delay)
+{
+  /* how many entries are in the array right now ? */
+  gint amount = 0;
+  while ( array != NULL && array[amount] != NULL ) amount++;
+  
+  Log** target = (Log**) new void*[amount+2];
+  amount = 0;
+  while ( array != NULL && array[amount] != NULL )
+    {
+      target[amount] = array[amount];
+      amount++;
     }
-  }
-  syslogs = fillups;
+  /* delete old array */
+  if ( array != NULL ) delete array;
+  
+  /* create new entry */
+  target[amount] = new Log();
+  target[amount+1] = NULL;
 
   /*
-    We don't need the temporaries anymore
+    Initialize new entry with the data we got
   */
-  delete mode;
-  delete filename;
-  delete help;
-  delete buttontext;
-  delete interval;
-  delete commandline;
-  delete filter;
-  delete filter_exec;
-  delete filter_mode;
-  fillups = 0;
+  target[amount]->init( filename,
+			help,
+			buttontext,
+			plugin,
+			mode,
+			interval,
+			delay );
+  
+  return target;
 }
-void 
-unsetup_logs()
+/*
+  This functions adds the filter specifications (strings & mode)
+  to the filter array of the last element in the elements
+  structure given as parameter.
+
+  This is done via just searching the last one and adding to
+  it via Log::add_filter
+
+  no return value.
+ */
+void
+add_filter_to_last(Log** elements, const gchar* string, gint mode, const gchar* execline)
 {
-  for ( int i = 0 ; i < syslogs ; i++ ){
-    remove_alert(i);                                              // clear alert routines & data
-    gtk_widget_destroy(entry[i]->button);
-    delete entry[i];
-  }
-  delete entry;
-  syslogs = 0;
-  entry = NULL;
+  Log* element = NULL;
+  gint x = 0;
+  
+  while ( elements != NULL && elements[x]!= NULL )
+    {
+      element = elements[x];
+      x++;
+    }
+
+  if ( element != NULL )
+    {
+      element->add_filter(string, mode, execline);
+    }
+
+}
+/*
+  This routine cleanly kills a whole array of
+  Log entries.
+  
+  The parameter is the address of the NULL terminated
+  Log class element array, there is no return value.
+  
+  The checks for array != NULL are being done so the
+  routine won't crash even if you give it a NULL
+  parameter... this makes sure we don't do stupid things
+  here.
+*/
+void
+destroy_log_array(Log** array)
+{
+  gint x = 0;
+  
+  while ( array != NULL && array[x] != NULL )
+    {
+      delete array[x];
+      x++;
+    }
+  
+  if ( array != NULL )
+    {
+      delete array;
+    }
+  
 }
 void 
 free_memory()
 {
-  unsetup_logs();
-  if ( load_path != NULL ) delete load_path;
-  if ( save_path != NULL ) delete save_path;
-  if ( log_path != NULL ) delete log_path;
-  if ( alert_sound != NULL ) delete alert_sound;
-  if ( notice_sound != NULL ) delete notice_sound;
-  if ( uniconify_sound != NULL ) delete uniconify_sound;
-  uniconify_sound = notice_sound = alert_sound = NULL;
 
+  destroy_buttons();
+
+  if ( entry != NULL ) destroy_log_array(entry);
+  entry = NULL;
+
+  if ( load_path != NULL )
+    {
+      delete load_path;
+    }
+  if ( save_path != NULL )
+    {
+      delete save_path;
+    }
+  if ( log_path != NULL )
+    {
+      delete log_path;
+    }
+  if ( alert_sound != NULL ) 
+    {
+      delete alert_sound;
+    }
+  if ( notice_sound != NULL )
+    {
+      delete notice_sound;
+    }
+  if ( uniconify_sound != NULL )
+    {
+      delete uniconify_sound;
+    }
+  uniconify_sound = notice_sound = alert_sound = NULL;
+  
 #if HAVE_MLOCKALL
   if ( lockmem == TRUE ) munlockall();
 #endif /* HAVE_MLOCKALL */
-
+  
   delete read_buffer;
   read_buffer = NULL;
-
-  delete case_fold;
+  
 }
 void 
 disable()
@@ -440,6 +665,11 @@ determine_filter_class(gint mode)
 
   return fclass;
 }
+/*
+
+  resorts the filter elements in an array non-destructive !
+  
+*/
 void 
 sort_filters(Filter** filter)
 {
@@ -448,79 +678,31 @@ sort_filters(Filter** filter)
   while ( filter[amount] != NULL ) amount++;
   if ( amount == 0 ) return;
 
-  Filter** tmp = new Filter*[amount+1];
+  Filter** tmp = (Filter**) new void*[amount+1];
   int i, j, k;
   i = k = 0;
   gint current_mode = 0;
-  while ( current_mode != -1 ){
-    j = 0;
-    while ( filter[j] != NULL ){
+  while ( current_mode != -1 )
+    {
+      j = 0;
+      while ( filter[j] != NULL ){
       if (( filter[j]->mode & CLASS0_MASK ) == current_mode )
 	tmp[k++] = filter[j];
       j++;
+      }
+      current_mode = class0_filters[i++];
     }
-    current_mode = class0_filters[i++];
-  }
   tmp[k] = NULL;
-
+  
   k = 0;
-  while ( tmp[k] != NULL ){
-    filter[k] = tmp[k];
-    k++;
-  }
-  filter[k] = NULL;
-
-  delete tmp;
-  return;
-}
-/*
-  This function disassembles the used REGEX buffers as
-  cleanly as possible. Since there is only one translation table
-  for all REGEX buffers this buffer is not freed.
-
-  Everything else is freed and set to zero before deleting the
-  REGEX buffer itself.
-  
-  I'd prefer to be able to use regfree, but it is a POSIX routine
-  and since it is commented out in rx.c it would only exist on
-  machines that do not need the libimport - making the whole
-  libimport obsolete... so I had to create this little function which
-  does exactly what regfree would have done... :-)
-*/
-void 
-disassemble_regex(re_pattern_buffer* regex)
-{
-  if ( regex != NULL )
+  while ( tmp[k] != NULL )
     {
-      /* set translate table to NULL */
-      regex->translate = NULL;
-      
-      /* free fastmap and set to zero if it exists */
-      if ( regex->fastmap != NULL )
-	{
-	  delete regex->fastmap;
-	  regex->fastmap = NULL;
-	  regex->fastmap_accurate = 0;
-	}
-
-      /* free syntax_parens if the buffer exists */
-      if ( regex->syntax_parens != NULL )
-	{
-	  delete regex->syntax_parens;
-	  regex->syntax_parens = NULL;
-	}
-      
-      /* free buffer and set to zero if it is used */
-      if ( regex->buffer != NULL && regex->allocated != 0 )
-	{
-	  free(regex->buffer);
-	  regex->buffer = NULL;
-	  regex->allocated = 0;
-	}
-      
-      delete regex;
+      filter[k] = tmp[k];
+      k++;
     }
+  filter[k] = NULL;
   
+  delete tmp;
   return;
 }
 /*}}}*/
@@ -537,7 +719,9 @@ usage()
   cout << "xlogmaster [-activate <n>] [-alertcolor <r,g,b>] ";
   cout << "[-audio <on/off>] [-audio-alert <filename>/<off>] ";
   cout << "[-audio-notice <filename>/<off>] [-audio-uniconify <filename>/<off>] ";
-  cout << "[-buffer <n>[bkm]] [-config <file>] [-fadeseconds <n>] [-fadesteps <n>] ";
+  cout << "[-buffer <n>[bkm]] ";
+  cout << "[-config <file>] [-database <directory>] ";
+  cout << "[-fadeseconds <n>] [-fadesteps <n>] ";
   cout << "[-geometry widthxheight+x+y] [-gtkrc <file>] ";
   cout << "[-help ] ";
 #if HAVE_MLOCKALL
@@ -656,13 +840,91 @@ get_value(char* string, int* index)
   else *index = y;
   return value;
 }
+/*
+  This function takes care that the paths for the configuration
+  and the database are being set correctly
+*/
+void
+evaluate_paths()
+{
+  /* evaluate compile-time settings */
+  config_dir = new gchar[ strlen(XLM_HOME)+2 ];
+  database_dir = new gchar[ strlen(XLM_DB)+2 ];
+  lib_dir = new gchar[ strlen(XLM_LIB)+2 ];
+  strcpy( config_dir, XLM_HOME );
+  strcpy( database_dir, XLM_DB );
+  strcpy( lib_dir, XLM_LIB );
+
+  /* fill in the path for the users private database */
+  gchar* hdir = NULL;
+  hdir = getenv("HOME");
+  if ( hdir == NULL ) hdir = getenv("USER");
+  if ( hdir == NULL ) hdir = getenv("PWD");
+  if ( hdir == NULL ) hdir = ".";
+  private_database_dir = 
+    new gchar[ strlen( hdir ) + strlen( XLM_USER_DB ) + 2 ];
+  strcpy( private_database_dir , hdir );
+  strcat( private_database_dir , "/" );
+  strcat( private_database_dir , XLM_USER_DB );
+
+  /* evaluate the environment variables */
+  gchar* env_conf = getenv( "XLM_HOME" );
+  gchar* env_db = getenv( "XLM_DB" );
+  gchar* env_user_db = getenv( "XLM_USER_DB" );
+  gchar* env_lib = getenv( "XLM_LIB" );
+
+  if ( env_conf != NULL )
+    {
+      delete config_dir;
+
+      config_dir = new gchar[ strlen(env_conf)+2 ];
+      strcpy( config_dir, env_conf );
+
+    }
+
+  if ( env_db != NULL )
+    {
+      delete database_dir;
+
+      database_dir = new gchar[ strlen(env_db)+2 ];
+      strcpy( database_dir, env_db );
+
+    }
+
+  if ( env_user_db != NULL )
+    {
+      delete private_database_dir;
+
+      private_database_dir = new gchar[ strlen(env_user_db)+2 ];
+      strcpy( private_database_dir, env_user_db );
+
+    }
+
+  if ( env_lib != NULL )
+    {
+      delete lib_dir;
+      
+      lib_dir = new gchar[ strlen(env_lib)+2 ];
+      strcpy( lib_dir, env_lib );
+      
+    }
+
+}
 /*}}}*/
 void 
 init(int argc,char* argv[])
 {
 /*{{{  Set startup defaults  */
+  /* we never want to get killed by sigpipes here, so
+     we rather ignore it for the whole Xlogmaster */
+  signal( SIGPIPE , SIG_IGN );
+
+  /* first set the configuration directories to sane defaults */
+  evaluate_paths();
+
+  entry = NULL; //
   syslogs = 0; // Nothing to display (yet)
-  fillups = 0; // When we enter here there is nothing to be filled up by definition
+
   request_active = 0; // By default the first log is being displayed  
   maxtext = MAXTEXT; // Default from config.H
   read_buffer = NULL;
@@ -680,13 +942,13 @@ init(int argc,char* argv[])
 
   /* for execute.cc: */
   cleanup_tag = pid_reference = 0;
-  pid_list = g_list_alloc();
+  pid_list = NULL;
 
   /* for audio.cc: */
-  alert_sound = new char[ strlen( STYLEDIR ) + strlen("/sound/") + 20 ];
-  notice_sound = new char[ strlen( STYLEDIR ) + strlen("/sound/") + 20 ];
-  uniconify_sound = new char[ strlen( STYLEDIR ) + strlen("/sound/") + 20 ];
-  strcpy(alert_sound, STYLEDIR);
+  alert_sound = new char[ strlen( lib_dir ) + strlen("/sound/") + 20 ];
+  notice_sound = new char[ strlen( lib_dir ) + strlen("/sound/") + 20 ];
+  uniconify_sound = new char[ strlen( lib_dir ) + strlen("/sound/") + 20 ];
+  strcpy(alert_sound, lib_dir);
   strcat(alert_sound, "/sound/");
   strcpy(notice_sound, alert_sound);
   strcpy(uniconify_sound, alert_sound);
@@ -706,17 +968,8 @@ init(int argc,char* argv[])
   eval_colorstring(cstring);
   delete cstring;
 
-
-  /*
-    set up translation table for case insensitivity
-  */
-  gint i;
-  case_fold = new gchar[257];
-  for ( i = 0 ; i < 256 ; i++ )
-    case_fold[i] = i;
-  for ( i = 'a' ; i <= 'z' ; i++ )
-    case_fold[i] = i - ( 'a' - 'A' );
-  case_fold[256] = 0;
+  /* dabatase menu form */
+  db_form = NULL;
 
 /*}}}*/
 
@@ -729,8 +982,8 @@ init(int argc,char* argv[])
  gchar* system_path = NULL;
  system_path = (gchar*) getenv("PATH");
  gchar* xlm_path;
- xlm_path = (gchar*) new char [ strlen( STYLEDIR ) + strlen( system_path ) + 20 ];
- strcpy(xlm_path, STYLEDIR);
+ xlm_path = (gchar*) new char [ strlen( lib_dir ) + strlen( system_path ) + 20 ];
+ strcpy(xlm_path, lib_dir);
  strcat(xlm_path, "/scripts:");
  strcat(xlm_path, system_path);
  my_setenv( "PATH", xlm_path);
@@ -814,8 +1067,14 @@ init(int argc,char* argv[])
    else if ( ! strcmp( opt, "config" ) )
      {
        if ( optarg == NULL ) usage();
-       fillups = 0;
-       read_configuration(optarg);
+       entry = read_configuration_file(optarg);
+     }
+   else if ( ! strcmp( opt, "database" ) )
+     {
+       if ( optarg == NULL ) usage();
+       delete database_dir;
+       database_dir = new gchar[ strlen(optarg) + 2 ];
+       strcpy( database_dir, optarg );
      }
    else if ( ! strcmp( opt, "fadeseconds" ) )
      {
@@ -936,8 +1195,8 @@ init(int argc,char* argv[])
  if ( gtkrc == FALSE )
    {
      char* gtkrc_fn;
-     gtkrc_fn = new char[ strlen( STYLEDIR ) + strlen( GTKRC ) + 3 ];
-     strcpy(gtkrc_fn, STYLEDIR);
+     gtkrc_fn = new char[ strlen( config_dir ) + strlen( GTKRC ) + 3 ];
+     strcpy(gtkrc_fn, config_dir);
      strcat(gtkrc_fn, "/");
      strcat(gtkrc_fn, GTKRC);
      gtk_rc_parse(gtkrc_fn);
@@ -949,7 +1208,7 @@ init(int argc,char* argv[])
  /*
    read only if we didn't read an explicit config file
  */
- if (  fillups == 0 )
+ if (  entry == NULL )
    {
      gchar* fname;
      gchar* hdir = NULL;
@@ -961,7 +1220,7 @@ init(int argc,char* argv[])
      strcpy ( fname, hdir );
      strcat ( fname, "/" );
      strcat ( fname, CONFIGFILE );
-     read_configuration(fname);
+     entry = read_configuration_file(fname);
      delete fname;
    }
 /*}}}*/
@@ -971,122 +1230,139 @@ init(int argc,char* argv[])
     read only if we didn't read an explicit or private
     config file
   */
- if (  fillups == 0 )
+ if (  entry == NULL )
    {
      gchar* fname;     
-     fname = new gchar[ strlen( STYLEDIR ) + strlen( SYSCONFIGFILE ) + 3 ];
-     strcpy ( fname, STYLEDIR );
+     fname = new gchar[ strlen( config_dir ) + strlen( SYSCONFIGFILE ) + 3 ];
+     strcpy ( fname, config_dir );
      strcat ( fname, "/" );
      strcat ( fname, SYSCONFIGFILE );
-     read_configuration(fname);
+     entry = read_configuration_file(fname);
      delete fname;
    }
 /*}}}*/
 
-/*{{{  Some logs for new users  */
-  if ( fillups == 0 ){
-    /*
-      Alright - since we don't want to pop up an empty window without
-      any boxes we better find something to display... :-)
+/*{{{  Some entries for new users  */
+  if ( entry == NULL )
+    {
+      /*
+	Alright - since we don't want to pop up an empty window without
+	any boxes we better find something to display... :-)
 
-      Some standards should be /var/log/messages,
-      /var/log/boot.msg, /var/adm/debug, so we'll give them for
-      starters since the user didn't give us anything else.
+	This also prevents a critical error in display_buttons() that
+	gets triggered if there is nothing to display...
+      */
 
-      For demonstration of the "cat" mode we also have a continuous
-      display of /proc/meminfo as a default
+
+      /*
+	/var/log/messages should be fairly standard
+      */
+      entry = add_log_entry( entry,
+			     TAIL_FILE,
+			     "/var/log/messages",
+			     "Display /var/log/messages file...",
+			     "System Messages",
+			     NULL,
+			     INTERVAL,
+			     6 );
+
+      /*
+	same for /var/log/boot.msg
+      */
+      entry = add_log_entry( entry,
+			     TAIL_FILE,
+			     "/var/log/boot.msg",
+			     "Display /var/log/boot.msg file...",
+			     "Bootup Messages",
+			     NULL,
+			     10000,
+			     6 );
       
-     */
+      /*
+	a nice one is also /proc/meminfo
+      */
+      entry = add_log_entry( entry,
+			     CAT_FILE,
+			     "/proc/meminfo",
+			     "Display memory usage via /proc/meminfo",
+			     "Memory Status",
+			     NULL,
+			     10,
+			     6 );
+      
+      /*
+	'df' should exist on all systems, too...
+      */
+      entry = add_log_entry( entry,
+			     RUN_FILE,
+			     "df",
+			     "Display Disk usage via running 'df'",
+			     "Harddisk Usage",
+			     NULL,
+			     3,
+			     15 );
 
-    fillups = 4;
-    mode = new int[fillups];
-    filename = new char*[fillups];
-    help = new char*[fillups];
-    buttontext = new char*[fillups];
-    interval = new int[fillups];
-    commandline = new char**[fillups];
-    filter = new char**[fillups];
-    filter_exec = new char**[fillups];
-    filter_mode = new int*[fillups];
-    for ( i = 0 ; i < fillups ; i++ ){
-      filename[i] = new char[50];
-      help[i] = new char[50];  
-      buttontext[i] = new char[50];
+      
+      /*
+	'w' isn't too exotic, either...
+      */
+      entry = add_log_entry( entry,
+			     RUN_FILE,
+			     "w",
+			     "Display user information by running 'w'",
+			     "Who's Online",
+			     NULL,
+			     3,
+			     10 );
+      
+      /*
+	'pstree' is pretty cool, as well...
+      */
+      entry = add_log_entry( entry,
+			     RUN_FILE,
+			     "pstree",
+			     "Display information about running processes via 'pstree'",
+			     "Process Tree",
+			     NULL,
+			     3,
+			     15 );
     }
-
-    /*
-      /var/log/messages
-    */
-    mode[0] = TAIL_FILE;
-    strcpy ( filename[0] , "/var/log/messages" );
-    strcpy ( help[0] , "Display /var/log/messages file..." );
-    strcpy ( buttontext[0] , "System Messages" );
-    interval[0] = INTERVAL;
-    commandline[0] = NULL;
-    filter[0] = NULL;
-    filter_exec[0] = NULL;
-    filter_mode[0] = NULL;
-
-    /*
-      /var/adm/debug
-    */
-    mode[1] = TAIL_FILE;
-    strcpy ( filename[1] , "/var/adm/debug");
-    strcpy ( help[1] , "Display /var/adm/debug file...");
-    strcpy (buttontext[1] , "Debugging Messages");
-    interval[1] = INTERVAL;
-    commandline[1] = NULL;
-    filter[1] = NULL;
-    filter_exec[1] = NULL;
-    filter_mode[1] = NULL;
-    
-    /*
-      /var/log/boot.msg
-    */
-    mode[2] = TAIL_FILE;
-    strcpy ( filename[2] , "/var/log/boot.msg");
-    strcpy (help[2] , "Display /var/log/boot.msg file...");
-    strcpy (buttontext[2] , "Bootup Messages");
-    interval[2] = INTERVAL;
-    commandline[2] = NULL;
-    filter[2] = NULL;
-    filter_exec[2] = NULL;
-    filter_mode[2] = NULL;
-
-    /*
-      /proc/meminfo
-    */
-    mode[3] = CAT_FILE;
-    strcpy ( filename[3] , "/proc/meminfo");
-    strcpy (help[3] , "Display memory usage via /proc/meminfo");
-    strcpy (buttontext[3] , "Memory Status");
-    interval[3] = 10; // show this one once a second no matter of
-    // config.H setting... 
-    commandline[3] = NULL;
-    filter[3] = NULL;
-    filter_exec[3] = NULL;
-    filter_mode[3] = NULL;
-    
-  }
 /*}}}*/
 
 #if HAVE_MLOCKALL
-// if user requests so, try to lock the xlogmaster in memory...
- if ( lockmem == TRUE ){
-   cout << endl << "Trying to lock the Xlogmaster in memory.... ";
-
+  // if user requests so, try to lock the xlogmaster in memory...
+  if ( lockmem == TRUE )
+    {
+      cout << endl << "Trying to lock the Xlogmaster in memory.... ";
+      
    int result = mlockall(MCL_FUTURE);  
 
-   if ( result == -1 ){
-     cout << "failed. Insufficient permission ?" << endl;
-     cout << "see the manpage for more information !" << endl;
-   } else {
-     cout << "success." << endl;
-     cout << "The Xlogmaster will not get swapped out." << endl;
-   }
- }
+   if ( result == -1 )
+     {
+       cout << "failed. Insufficient permission ?" << endl;
+       cout << "see the manpage for more information !" << endl;
+     } 
+   else 
+     {
+       cout << "success." << endl;
+       cout << "The Xlogmaster will not get swapped out." << endl;
+     }
+    }
 #endif /* HAVE_MLOCKALL */
- read_buffer = new char[maxtext+2];
+  
+  /*
+    check out the buffer size so it doesn't get bigger
+    than SSIZE_MAX
+  */
+  if ( maxtext > SSIZE_MAX)
+    {
+      maxtext = SSIZE_MAX;
+    }
+  
+  /*
+    allocate buffer
+  */
+  read_buffer = new char[maxtext+2];
 }
 /*}}}*/
 
@@ -1137,14 +1413,18 @@ output_text(GtkWidget* target, char* buffer, long size)
   while ( stop < size )
     {
    
-      new_stop = match_all( entry[active]->filter, CLASS0_FILTER, buffer, stop, size);
+      new_stop = match_all( entry[active]->filter, 
+			    CLASS0_FILTER, 
+			    buffer, 
+			    stop, 
+			    size);
 
       gint action = NO_CHANGE;
       gint i = 0;
       while ( entry[active]->filter[i] != NULL )
 	{
 	  
-	  if ( entry[active]->filter[i]->regex_status >= 0 )
+	  if ( entry[active]->filter[i]->regex_status == 0 )
 	    {
 	      action |= entry[active]->filter[i]->mode;
 	    }
@@ -1226,8 +1506,8 @@ output_text(GtkWidget* target, char* buffer, long size)
     output the remaining normal text (no special matches found)
   */
   if ( start != stop )
-    gtk_text_insert (GTK_TEXT(target),NULL,NULL,NULL,
-		     (char*) (buffer+start), (stop-start));
+    gtk_text_insert ( GTK_TEXT(target),NULL,NULL,NULL,
+		      (char*) (buffer+start), (stop-start) );
   
 
 }
@@ -1266,8 +1546,8 @@ match_all(Filter** filter, gint want_class, gchar* buffer, glong index, glong si
   /*
     now go to beginning of next one
   */
-  while ( buffer[end] != 0 && end < size &&
-	  buffer[end] == '\n' && buffer[end] == '\r' ) end++;
+  while ( buffer[end+1] != 0 && ( end + 1 ) < size &&
+	  ( buffer[end+1] == '\n' || buffer[end+1] == 10 ) ) end++;
   
   /* allocate memory */
   line = new gchar[ end - index + 5 ];
@@ -1309,17 +1589,18 @@ match_all(Filter** filter, gint want_class, gchar* buffer, glong index, glong si
 	}
       else
 	{
-	  filter[x]->regex_status = -1;
+	  filter[x]->regex_status = REG_NOMATCH;
 	}
-
+      
       x++;
     }
-
+  
   /*
     alright, that's it... we're done. Free string element "line" and return
     our end position.
   */
   delete line;
+
   return end;
 }
 /*
@@ -1339,22 +1620,21 @@ match_line(Filter* filter, gchar* line)
 {
   if ( filter->regex_pattern == NULL )
     {
-      return -1;
+      return REG_NOMATCH;
     }
 
-  gint result = re_search( filter->regex_pattern,
-			   line,
-			   strlen( line ),
-			   0,
-			   strlen( line ),
-			   NULL );
+  gint result = regexec( filter->regex_pattern,
+			 line,
+			 0,
+			 NULL,
+			 0 );
   
   if ( filter->mode & INVERT )
-    {      
-      if ( result >= 0 )
-	result = -1;
+    {
+      if ( result == 0 )
+	result = REG_NOMATCH;
       else
-	result = 0; 
+	result = 0;
     }
   
   return result;
@@ -1370,8 +1650,8 @@ file_error(gchar* file)
   cerr << "  " << file << endl;
 
   /* and now popup dialog */
-  gchar* opener = "\n\t     Error while trying to access file     \t\n\n\t      ";
-  gchar* closer = "      \t\n\n\t   please check path and permissions.     \t\n";
+  gchar* opener = "\n      Error while trying to access file      \n\n       ";
+  gchar* closer = "       \n\n    please check path and permissions.      \n";
   gchar* text = new gchar[strlen(file) + strlen(opener) + strlen(closer) + 10 ];
   strcpy ( text, opener );
   strcat ( text, file );
@@ -1380,7 +1660,7 @@ file_error(gchar* file)
   GList* buttons = g_list_alloc();
   GList* functions = g_list_alloc();
   buttons = g_list_append(buttons,
-			  (gpointer) "\t   DISMISS   \t");
+			  (gpointer) "    DISMISS    ");
   functions = g_list_append(functions, NULL );
 
   popup_dialog("File Access Error", text, buttons, functions);
@@ -1396,11 +1676,62 @@ fork_error()
   GList* buttons = g_list_alloc();
   GList* functions = g_list_alloc();
   buttons = g_list_append(buttons,
-			  (gpointer) "\t   DISMISS   \t");
+			  (gpointer) "    DISMISS    ");
   functions = g_list_append(functions, NULL );
   
-  popup_dialog("Fork Error", "\n\t    Error while trying to spawn a new process !!!     \t\n", buttons, functions);
+  popup_dialog("Fork Error", "\n     Error while trying to spawn a new process !!!      \n", buttons, functions);
 }
+void 
+execute_error(gchar* file)
+{
+  /* give stderr message */
+  cerr << PACKAGE << ": execution error. Could not run" << endl;
+  cerr << "  " << file << endl;
+  cerr << "  " << "due to fork-problems." << endl;
+
+  /* and now popup dialog */
+  gchar* opener = "\n      Error while trying to run program      \n\n       ";
+  gchar* closer = "       \n\n    due to fork problems. Insufficient memory ?      \n";
+  gchar* text = new gchar[strlen(file) + strlen(opener) + strlen(closer) + 10 ];
+  strcpy ( text, opener );
+  strcat ( text, file );
+  strcat ( text, closer );
+
+  GList* buttons = g_list_alloc();
+  GList* functions = g_list_alloc();
+  buttons = g_list_append(buttons,
+			  (gpointer) "    DISMISS    ");
+  functions = g_list_append(functions, NULL );
+  
+  popup_dialog("Execution Error", text, buttons, functions);
+  delete text;
+}
+void 
+plugin_overflow(gchar* file)
+{
+  /* give stderr message */
+  cerr << PACKAGE << ": plugin pipe overflow for pipe into" << endl;
+  cerr << "  " << file << endl;
+
+  /* and now popup dialog */
+  gchar* opener = "\n      Error in pipe into plugin      \n\n       ";
+  gchar* closer = "       \n\n    ,the input pipe had a buffer overflow.   \n";
+  gchar* text = new gchar[strlen(file) + strlen(opener) + strlen(closer) + 10 ];
+  strcpy ( text, opener );
+  strcat ( text, file );
+  strcat ( text, closer );
+
+  GList* buttons = g_list_alloc();
+  GList* functions = g_list_alloc();
+  buttons = g_list_append(buttons,
+			  (gpointer) "    DISMISS    ");
+  functions = g_list_append(functions, NULL );
+  
+  popup_dialog("Plugin Error", text, buttons, functions);
+  delete text;
+}
+
+
 
 /*}}}*/
 
@@ -1409,7 +1740,7 @@ void
 create_about()
 {
   GtkWidget* button;
-  about_dialog = gtk_window_new( GTK_WINDOW_DIALOG);
+  about_dialog = gtk_window_new( GTK_WINDOW_DIALOG );
   gtk_signal_connect (GTK_OBJECT (about_dialog), "destroy",
 		      GTK_SIGNAL_FUNC (about_ready), NULL);
   gtk_signal_connect (GTK_OBJECT (about_dialog), "delete_event",
@@ -1477,7 +1808,7 @@ create_about()
 		      ,TRUE, TRUE, 0);
   gtk_widget_show (label);
 
-  label = gtk_label_new ( " \n \t \t Copyright (C) 1998 Georg C. F. Greve \n \t \t This is a GNU program \n \t \n \t This program is distributed in the hope that it will be useful, \t \n but WITHOUT ANY WARRANTY; without even the implied warranty of \n MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n GNU General Public License for more details. \n \t \t \n \n If you experience any problems, discover any bugs, \n think some new feature would be great or just want to say \n how much you like this program, write an email to: \n \n \t xlogmaster-bugs@gnu.org \t \n \n \n Thanks to the GTK+ crew for writing this great toolkit, I like \n it a lot - and to my girlfriend Julia for liking me even when \n I'm in a coding frenzy & for the great logo... ;-) \n");
+  label = gtk_label_new ( " \n     Copyright (C) 1998 Georg C. F. Greve \n     This is a GNU program \n   \n   This program is distributed in the hope that it will be useful,   \n but WITHOUT ANY WARRANTY; without even the implied warranty of \n MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n GNU General Public License for more details. \n     \n \n If you experience any problems, discover any bugs, \n think some new feature would be great or just want to say \n how much you like this program, write an email to: \n \n   xlogmaster-bugs@gnu.org   \n \n \n Thanks to the GTK+ crew for writing this great toolkit, I like \n it a lot - and to my girlfriend Julia for liking me even when \n I'm in a coding frenzy & for the great logo... ;-) \n");
   gtk_box_pack_start (GTK_BOX (vbox), label
 		      ,TRUE, TRUE, 0);
   gtk_widget_show (label);
